#include <stdio.h>
#include <stdlib.h>
#include <math.h>
#include <iostream>
#include <fstream>
#include <sstream>
#include <string.h>
#include <time.h>
using namespace std;

#define max_length 200000						//maximum sequence length
#define bin_size 25.0							//bin size used in Student t distribution (prior probability)

int name_input_sequences;						//name of input sequence file: FASTA format
int name_input_basecoverage;					//name of input sequencing depth file: HPeak output format
int name_input_peaklocation;					//name of input peak summit location file: the location with highest sequencing depth 
int flag_basecoverage;							//an indicator: 0 -- no base coverage information, uniform prior; 1 -- informative prior proportional to base coverage; 2 -- informative prior (Student t distribution centered at the peak summit)
int flag_strand;								//an indicator: 1 -- scan only the input strand; 2 -- scan both the input strand and the reversed strand
int flag_triple;								//an indicator: -1 -- intra-motif independency; 0 -- up to pair intra-motif dependency; 1 -- up to triple intra-motif dependency
int K;											//double number of sequence 
int Khalf;										//number of sequence  
int w;											//length of motif
int d;											//d=4 for nucletide sequence a, c, g, t; d=20 for amino acid sequences
int chain;										//number of parallel chains
int LOOP;										//number of iteration in each chain
double seqprop;									//percentage of sequences used in stochastic gibbs sampler
int countchain;									//count the index of parallel chain       
double t_dof;									//degree of freedom in Student t distribution (informative prior)

double **rowscore;								//ratio between motif model and background model at each location, used for stochastic sampler or greedy search
double **logplot;								//overall log motif ratio, used for trace plot and check convergenece
int    *halfseqlength,*seqlength;				//count the length of each sequence
int    **r,**rinput,**rinverse;					//input strand, reverse strand, combined double strands
double **rbase, **rbaseinput, **rbaseinverse;	//sequencing depth
int    *rpeak, *rpeakinput, *rpeakinverse;		//peak summit location 
double **rbackground;							//same size as r, calculated by 3-order Markov background
double *theta0, **theta1, **theta2, **theta3;	//3-order Markov background for a c g t
double **theta, **thetatemp, **thetafinal;		//PSWM d*w matrix
int    *a, *atemp, *afinal;						//motif starting position (alignment variable)
int    *id, *idtemp, *idfinal;					//id=1 indicates that sequence contains motif, id=0 otherwise
long   **IX;									//index, used in generate multi-nomial distribution
long   *SAMPLE;									//indicator: which seqeunce will be chosen to stochastic search
double **sampleprob;							//probability of being sampled for gibbs, LOOP*K matrix

int K_motif;																	//number of identified motif
int ** r_motif;																	//a collection of identified motif: K_motif*w matrix
int ** motif_model, ** motif_model_temp, ** motif_model_final;					//w*4 matrix: record up to triple intra-motif dependency
double **r_theta_single, **r_theta_single_temp, **r_theta_single_final;			//d * w matrix for single base: multinomial(p1,...,p4) = (pA,...,pT)
double ***r_theta_pair, ***r_theta_pair_temp, ***r_theta_pair_final;			//(d*d)*w*w matrix for base pair: multinomial(p1,...,p16) = (pAA,...,pTT) 
double ****r_theta_triple, ****r_theta_triple_temp,****r_theta_triple_final;	//(d*d*d)*w*w matrix for base triple: multinomial(p1,...,p64) = (pAAA,...,pTTT) 

//partition function used in quick sort
int partition(double a[], int start, int stop, int id[])
{
 int temp_id, up=start, down=stop-1;
 double temp_value, part=a[stop];
 if(stop<=start) return start;

 while(true)
 {
  while(a[up]<part) up++;
  while(part<a[down] && (up<down)) down--;

  if(up>=down) break;

  temp_value=a[up];  a[up]=a[down]; a[down]=temp_value;
  temp_id=id[up]; id[up]=id[down]; id[down]=temp_id;

  up++; down--;
 }

 temp_value=a[up]; a[up]=a[stop]; a[stop]=temp_value;
 temp_id=id[up]; id[up]=id[stop]; id[stop]=temp_id;
 return up;
}

//quick sort 
void quickSort(double a[], int start, int stop, int id[])
{
 int i;
 if(stop<=start) return;

 i=partition(a,start,stop,id); 
 quickSort(a,start,i-1,id);
 quickSort(a,i+1,stop,id);
}

/* Period parameters */  
#define N 624
#define M 397
#define MATRIX_A 0x9908b0dfUL   /* constant vector a */
#define UPPER_MASK 0x80000000UL /* most significant w-r bits */
#define LOWER_MASK 0x7fffffffUL /* least significant r bits */

static unsigned long mt[N]; /* the array for the state vector  */
static int mti=N+1; /* mti==N+1 means mt[N] is not initialized */

/* initializes mt[N] with a seed */
void init_genrand(unsigned long s)
{
    mt[0]= s & 0xffffffffUL;
    for (mti=1; mti<N; mti++) {
        mt[mti] = 
            (1812433253UL * (mt[mti-1] ^ (mt[mti-1] >> 30)) + mti); 
        /* See Knuth TAOCP Vol2. 3rd Ed. P.106 for multiplier. */
        /* In the previous versions, MSBs of the seed affect   */
        /* only MSBs of the array mt[].                        */
        /* 2002/01/09 modified by Makoto Matsumoto             */
        mt[mti] &= 0xffffffffUL;
        /* for >32 bit machines */
    }
}

/* initialize by an array with array-length */
/* init_key is the array for initializing keys */
/* key_length is its length */
/* slight change for C++, 2004/2/26 */
void init_by_array(unsigned long init_key[], int key_length)
{
    int i, j, k;
    init_genrand(19650218UL);
    i=1; j=0;
    k = (N>key_length ? N : key_length);
    for (; k; k--) {
        mt[i] = (mt[i] ^ ((mt[i-1] ^ (mt[i-1] >> 30)) * 1664525UL))
          + init_key[j] + j; /* non linear */
        mt[i] &= 0xffffffffUL; /* for WORDSIZE > 32 machines */
        i++; j++;
        if (i>=N) { mt[0] = mt[N-1]; i=1; }
        if (j>=key_length) j=0;
    }
    for (k=N-1; k; k--) {
        mt[i] = (mt[i] ^ ((mt[i-1] ^ (mt[i-1] >> 30)) * 1566083941UL))
          - i; /* non linear */
        mt[i] &= 0xffffffffUL; /* for WORDSIZE > 32 machines */
        i++;
        if (i>=N) { mt[0] = mt[N-1]; i=1; }
    }

    mt[0] = 0x80000000UL; /* MSB is 1; assuring non-zero initial array */ 
}

/* generates a random number on [0,0xffffffff]-interval */
uint32_t genrand_int32(void) {
  unsigned long y;
  static unsigned long mag01[2]={0x0UL, MATRIX_A};
  /* mag01[x] = x * MATRIX_A  for x=0,1 */

  if (mti >= N) { /* generate N words at one time */
    int kk;

    if (mti == N+1)   /* if init_genrand() has not been called, */
      init_genrand(5489UL); /* a default initial seed is used */

    for (kk=0;kk<N-M;kk++) {
      y = (mt[kk]&UPPER_MASK)|(mt[kk+1]&LOWER_MASK);
      mt[kk] = mt[kk+M] ^ (y >> 1) ^ mag01[y & 0x1UL];
    }
    for (;kk<N-1;kk++) {
      y = (mt[kk]&UPPER_MASK)|(mt[kk+1]&LOWER_MASK);
      mt[kk] = mt[kk+(M-N)] ^ (y >> 1) ^ mag01[y & 0x1UL];
    }
    y = (mt[N-1]&UPPER_MASK)|(mt[0]&LOWER_MASK);
    mt[N-1] = mt[M-1] ^ (y >> 1) ^ mag01[y & 0x1UL];

    mti = 0;
  }
  
  y = mt[mti++];

  /* Tempering */
  y ^= (y >> 11);
  y ^= (y << 7) & 0x9d2c5680UL;
  y ^= (y << 15) & 0xefc60000UL;
  y ^= (y >> 18);

  return y;
}

uint64_t genrand_int64(void) {
  uint64_t x, y;

  x = genrand_int32(); y = genrand_int32();
  return (x<<32)|y;
}

void init_mersenne(void) {
  unsigned long init[4]={0x123, 0x234, 0x345, 0x456}, length=4;
  init_by_array(init, length);
}

double genrand_real2(void)
{
 return genrand_int64()*(1.0/(4294967296.0*4294967296.0)); 
}

double ranf(void)
{    
  init_genrand((unsigned)time(NULL)+rand());
  return (genrand_real2());
}

double genunf(double low,double high)
{
	static double genunf;
    genunf = low+(high-low)*ranf(); 

    return genunf;
}

//generate a random vector follows multinomial distribution, used in prioritized hybrid monte carlo sampling
void genmul(long n,double *p,long ncat,long *ix)
{
int i,j;
double indicate;

double * q = NULL;
q = new double [ncat];

q[0]=p[0];
for(i=1;i<ncat;i++) q[i]=p[i]+q[i-1];

for(i=0;i<ncat;i++) ix[i]=0;

for(i=0;i<n;i++)
{
 indicate=genunf(0,1); 
 if(indicate<q[0]) ix[0]++;
 else if(indicate>=q[ncat-1]) ix[ncat-1]++;
 else
	 {
      for(j=1;j<ncat;j++)
	  {
	   if(indicate>=q[j-1] && indicate<q[j]) {ix[j]++; break;}	
	  }
	 }
}

return;
}

//generate a random variable (scalar) follows multinomial distribution, used to sample motif starting location (alignment variable)
int genmulone(double *p_input,long ncat)
{
int i;
double *p = NULL;
p = new double [ncat];
for(i=0;i<ncat;i++) p[i]=p_input[i];

int*q = NULL;
q = new int [ncat];
for(i=0;i<ncat;i++) q[i]=i;

quickSort(p, 0, ncat-1, q);

double prob=genunf(0,1);
int outcome=ncat-1;
while (outcome>=0  && prob > p[outcome])
{
 prob -= p[outcome];
 outcome--;
}

return q[outcome];
}

// set sample probability for each sequence in each iteration, the bias of sampling top sequence graduatelly diminish
// first row: K, K-1, ..., 1
// last row : K/2, K/2, ..., K/2    (equal chance being selected)
void setsampleprob()
{
 int i,j;
 double rowsum[LOOP];

 for(i=0;i<LOOP;i++)
 {
  for(j=0;j<K;j++) sampleprob[i][j] = K-j - (K/2-j)*i/(LOOP-1);
 }

 for(i=0;i<LOOP;i++)
 {
  rowsum[i]=0.0;
  for(j=0;j<K;j++) rowsum[i] += sampleprob[i][j];
 }

 for(i=0;i<LOOP;i++)
 {
  for(j=0;j<K;j++) sampleprob[i][j] /= rowsum[i];
 }

 for(i=0;i<LOOP;i++)
 {
  rowsum[i]=0.0;
  for(j=0;j<K;j++)
  {
   if(rowsum[i]<0.9999) rowsum[i] += sampleprob[i][j];
   else if(rowsum[i]>=0.9999) sampleprob[i][j]=0.0;
  }
 }

 return;
}

//calculate log motif ratio assuming that all positions in the motif are independent
double score(int **r, double **theta, int i, int j)
{
  int k;
  double sum=0.0;
  for(k=0;k<w;k++) sum += log( theta[ r[i][j+k] ][k]/ rbackground[i][j+k] + 10e-8); 
  return(sum);
}

//calculate log motif ratio assuming up to triple intra-motif depenedency 
double score_dep(int **r, double **theta, int i, int j, int **motif_model,
                 double **r_theta_single, double ***r_theta_pair, double ****r_theta_triple)
{
  int k;
  double sum=0.0;
  for(k=0;k<w;k++)  
	{     
     if(motif_model[k][0]==1)
       sum += log( r_theta_single[ r[i][j+k] ][k]/rbackground[i][j+k] +10e-8); 
	 else if(motif_model[k][0]==2)
       sum += log( r_theta_pair[ d*r[i][ j+motif_model[k][1] ]+r[i][ j+motif_model[k][2] ] ][ motif_model[k][1] ][ motif_model[k][2] ]
	 	           /(rbackground[i][ j+motif_model[k][1] ]*rbackground[i][ j+motif_model[k][2] ]) +10e-8);	 
	 else if(motif_model[k][0]==3)
	   sum += log( r_theta_triple[ d*d*r[i][ j+motif_model[k][1] ]+d*r[i][ j+motif_model[k][2] ]+r[i][ j+motif_model[k][3] ]  ][ motif_model[k][1] ][ motif_model[k][2] ][ motif_model[k][3] ]
	 	           /(rbackground[i][ j+motif_model[k][1] ]*rbackground[i][ j+motif_model[k][2] ]*rbackground[i][ j+motif_model[k][3] ])  +10e-8);	 
	 
	}
  return(sum);
}

//calculate log motif ratio for a single column, used in one base shift with Metropolis-Hasting rule
double score_single(int **r, int *position)
{
  int i,j;
  double sum=0.0, theta_single[d];
  for(i=0;i<d;i++) theta_single[i]=0.0;

  for(i=0;i<K;i++)
	{
     if(id[i]==1)
	  {
	   theta_single[ r[i][position[i]] ]++;
	  }
    }

  for(i=0;i<d;i++) sum += theta_single[i];
  for(i=0;i<d;i++) theta_single[i] /= sum;

  for(i=0;i<K;i++)
  {
   if(id[i]==1)
	{
     sum += log( theta_single[ r[i][position[i]] ]/ rbackground[i][position[i]] + 10e-8);
    } 
  }

  return(sum);
}

//randomly select motif starting position in each seqeuence (initialization with non-informative prior)
void setlabel()
{
  int i;
  double indicate;
  for(i=0;i<K;i++)
  {
     indicate=genunf(0,seqlength[i]-w+1);
     a[i]=(int)floor(indicate);
  }
  return;
}

//sample motif starting position in each seqeuence, which is proportional to sequencing depth profile (initialization with multinomial informative prior)
void setlabel_basecoverage()
{
  int i,j,k;
  double sump;

  for(i=0;i<K;i++)
  {
     sump=0.0;
	 for(j=0;j<seqlength[i]-w+1;j++) sump += rbase[i][j];
	 for(j=0;j<seqlength[i]-w+1;j++) rbase[i][j] /= sump;
     
	 for(j=0;j<seqlength[i]-w+1;j++) 
	 {
	  if(rbase[i][j]<0.0001) rbase[i][j]=0.0;
	  else if(rbase[i][j]>=0.0001) rbase[i][j] -= 0.0001;
	 }     

     a[i]=genmulone(rbase[i], seqlength[i]-w+1);
  }

  return;
}

//sample motif starting position in each seqeuence, which is proportional to rescale Student t distribution (initialization with informative prior)
void setlabel_peaklocation()
{
 int i,j,k;
 double sump;
 double ** rpeaktmp;

 rpeaktmp = new double*[K];
 for(i=0;i<K;i++) { rpeaktmp[i] = new double[ seqlength[i] ]; }

 for(i=0;i<K;i++)
	{
     for(j=0;j<seqlength[i];j++)
		{
		 rpeaktmp[i][j] = exp( -(t_dof+1)/2 * log( 1+ floor(fabs((j-rpeak[i])/bin_size)) * floor(fabs((j-rpeak[i])/bin_size)) /t_dof ) );
	    }
    }

 for(i=0;i<K;i++)
    {
     sump=0.0;
	 for(j=0;j<seqlength[i]-w+1;j++) sump += rpeaktmp[i][j];
	 for(j=0;j<seqlength[i]-w+1;j++) rpeaktmp[i][j] /= sump;
     
	 for(j=0;j<seqlength[i]-w+1;j++) 
	 {
	  if(rpeaktmp[i][j]<0.0001) rpeaktmp[i][j]=0.0;
	  else if(rpeaktmp[i][j]>=0.0001) rpeaktmp[i][j] -= 0.0001;
	 }     

     a[i]=genmulone(rpeaktmp[i], seqlength[i]-w+1);
    }

 return;
}

//find the reverse strand of input sequence
void setrinverse(int **rinput)
{
  int i,j,tmp;

  for(i=0;i<Khalf;i++)
	{
      for(j=0;j<halfseqlength[i];j++) rinverse[i][j]=(d-1)-rinput[i][j];  // a->0 c->1 g->2 t->3  	    
    }

  for(i=0;i<Khalf;i++)
	{
	  if(halfseqlength[i]%2==0)
	  {
       for(j=0;j<halfseqlength[i]/2;j++)
		{
	     tmp=rinverse[i][j];rinverse[i][j]=rinverse[i][halfseqlength[i]-j-1];rinverse[i][halfseqlength[i]-j-1]=tmp;
	    }
	  }
	  else if(halfseqlength[i]%2==1)
	  {
	   for(j=0;j<(halfseqlength[i]-1)/2;j++)
	    {
         tmp=rinverse[i][j];rinverse[i][j]=rinverse[i][halfseqlength[i]-j-1];rinverse[i][halfseqlength[i]-j-1]=tmp;
	    }
	  }
    }
  return;
}

//find the sequencing depth profile of the reverse strand of input sequence
void setrbaseinverse(double **rbaseinput)
{
  int i,j;
  double tmp;

    for(i=0;i<Khalf;i++)
	{
      for(j=0;j<halfseqlength[i];j++) rbaseinverse[i][j]=rbaseinput[i][j];    
    }

  for(i=0;i<Khalf;i++)
	{
	  if(halfseqlength[i]%2==0)
	  {
       for(j=0;j<halfseqlength[i]/2;j++)
		{
	     tmp=rbaseinverse[i][j];
		 rbaseinverse[i][j]=rbaseinverse[i][halfseqlength[i]-j-1];
		 rbaseinverse[i][halfseqlength[i]-j-1]=tmp;
	    }
	  }
	  else if(halfseqlength[i]%2==1)
	  {
	   for(j=0;j<(halfseqlength[i]-1)/2;j++)
	    {
         tmp=rbaseinverse[i][j];
		 rbaseinverse[i][j]=rbaseinverse[i][halfseqlength[i]-j-1];
		 rbaseinverse[i][halfseqlength[i]-j-1]=tmp;
	    }
	  }
    }
  return;
}

//find the peak summit for the reverse strand of input sequence
void setpeakinverse(int *rpeakinput)
{
 int i;
 for(i=0;i<Khalf;i++) rpeakinverse[i]=halfseqlength[i]-rpeakinput[i]-1;
 return;
}

//combine the input sequence and the reverse strand
void combine(int **rinput, int **rinverse)
{
  int i,j;
  
  for(i=0;i<Khalf;i++)
	{
      for(j=0;j<halfseqlength[i];j++)
		{
	      r[i*2][j]=rinput[i][j];
		  r[i*2+1][j]=rinverse[i][j];
	    }
    }
  return;
}

//combine the sequencing depth information of the input sequence and the reverse strand
void combinebase(double **rbaseinput, double **rbaseinverse)
{
  int i,j;
  
  for(i=0;i<Khalf;i++)
	{
      for(j=0;j<halfseqlength[i];j++)
		{
	      rbase[i*2][j]=rbaseinput[i][j];
		  rbase[i*2+1][j]=rbaseinverse[i][j];
	    }
    }
  return;
}

//combine the peak summit information of the input sequence and the reverse strand
void combinepeak(int *rpeakinput, int *rpeakinverse)
{
 int i;
 for(i=0;i<Khalf;i++)
	{
     rpeak[i*2]=rpeakinput[i];
	 rpeak[i*2+1]=rpeakinverse[i];
    }
 return;
}

//given the motif starting position a, calculate the motif PSWM, assume each seqeunce contains at least one motif, and assume intra-motif independency
void settheta(int **r, int *a)
{
 int i, j;
 double count_sum, count[d];
 
 for(j=0;j<w;j++)
 {
  count_sum=0.0;
  for(i=0;i<d;i++) count[i]=0.0;

  for(i=0;i<K;i++)
  {
    count[ r[i][a[i]+j] ]++;
  }
  
  for(i=0;i<d;i++) count_sum += count[i];
  for(i=0;i<d;i++) theta[i][j]=count[i]/count_sum;
 }
 return;
}

//given the motif starting position a, calculate the motif PSWM, drop the sequences which do not contain motif, and assume intra-motif independency
//update id[i], which indicates which seqeunce contains motif
void setthetadrop(int **r, int *a, int *id, double **theta)
{
 int i, j;
 double count_sum, count[d];
 double avg[K];

 for(i=0;i<K;i++)
 {
  avg[i]=0.0;

  for(j=0;j<seqlength[i]-w+1;j++)
  {
   avg[i] += exp(score(r,theta,i,j));
  }
   avg[i]=log(avg[i]/seqlength[i]);

   if(avg[i]>=0) id[i]=1;
   else if(avg[i]<0) id[i]=0;
 }

 for(j=0;j<w;j++)
 {
  count_sum=0.0;
  for(i=0;i<d;i++) count[i]=0.0;

  for(i=0;i<K;i++)
  {
	if(id[i]==1)
	{
	 count[ r[i][a[i]+j] ]++;
	}
  }

  for(i=0;i<d;i++) count_sum += count[i];
  for(i=0;i<d;i++) theta[i][j]=count[i]/count_sum;
 }
 return;
}

//return log emission prob for a single base, used in BIC calculation
double emissionindep(int **r_motif, int col) 
{
 int i,count[d];
 double value=0.0;
 
 for(i=0;i<d;i++) count[i]=0;
 for(i=0;i<K_motif;i++) count[ r_motif[i][col] ]++;
 for(i=0;i<d;i++) 
	{
	 if(r_theta_single[i][col]!=0) value += count[i]*log(r_theta_single[i][col]); 
    }

 return(value);
}

//return log emission prob for a base pair (col_i > col_j), used in BIC calculation
double emissioncor(int **r_motif, int col_i, int col_j) 
{
 int i,count[d*d];
 double value=0.0;

 for(i=0;i<d*d;i++) count[i]=0;
 for(i=0;i<K_motif;i++) count[ 4*r_motif[i][col_i]+r_motif[i][col_j] ]++;
 for(i=0;i<d*d;i++)
	{
	 if(r_theta_pair[i][col_i][col_j]!=0) value += count[i]*log(r_theta_pair[i][col_i][col_j]); 
    }

 return(value);
}

//return log emission prob for a base triple (col_i > col_j > col_k), used in BIC calculation
double emissiontriple(int **r_motif, int col_i, int col_j, int col_k) 
{
 int i,count[d*d*d];
 double value=0.0;

 for(i=0;i<d*d*d;i++) count[i]=0;
 for(i=0;i<K_motif;i++) count[ 16*r_motif[i][col_i]+4*r_motif[i][col_j]+r_motif[i][col_k] ]++;
 for(i=0;i<d*d*d;i++)
	{
	 if(r_theta_triple[i][col_i][col_j][col_k]!=0) value += count[i]*log(r_theta_triple[i][col_i][col_j][col_k]); 
    }

 return(value);
}

//count the percentage of "A, C, G, T" in each base, update d*w matrix r_theta_single
void count_theta_single(int ** r_motif)
{
 int i,j;

 for(j=0;j<w;j++)
  {
   for(i=0;i<d;i++) r_theta_single[i][j]=0.0;  
   for(i=0;i<K_motif;i++) r_theta_single[ r_motif[i][j] ][j]++;
  } 

 for(j=0;j<w;j++)
  {
   for(i=0;i<d;i++) r_theta_single[i][j]=r_theta_single[i][j]/K_motif;
  } 

 return;
}

//count the percentage of "AA, AC, ..., TT" in each base pair, update (d*d)*w*w matrix r_theta_pair
void count_theta_pair(int ** r_motif)
{
 int i,j,k;

 for(i=0;i<d*d;i++)
    {
     for(j=0;j<w-1;j++) 
	    {
	     for(k=j+1;k<w;k++) r_theta_pair[i][j][k]=0.0;
	    }
    }
 
 for(i=0;i<w-1;i++)
	{
     for(j=i+1;j<w;j++)
		{
	     for(k=0;k<K_motif;k++) r_theta_pair[ 4*r_motif[k][i]+r_motif[k][j] ][i][j]++;       
	    }
    }
 
 for(i=0;i<d*d;i++)
    {
     for(j=0;j<w-1;j++) 
	    {
	     for(k=j+1;k<w;k++) r_theta_pair[i][j][k]=r_theta_pair[i][j][k]/K_motif;
	    }
    }

 return;
}

//count the percentage of "AAA, AAC, ..., TTT" in each base triple, update (d*d*d)*w*w*w matrix r_theta_triple
void count_theta_triple(int ** r_motif)
{
 int i,j,k,l;

 for(i=0;i<d*d*d;i++)
    {
     for(j=0;j<w-2;j++) 
	    {
	     for(k=j+1;k<w-1;k++) 
			{
			 for(l=k+1;l<w;l++) r_theta_triple[i][j][k][l]=0.0;
		    }
	    }
    }
 
 for(i=0;i<w-2;i++)
	{
     for(j=i+1;j<w-1;j++)
		{
	     for(k=j+1;k<w;k++)
		 {
    	  for(l=0;l<K_motif;l++) r_theta_triple[ 16*r_motif[l][i]+4*r_motif[l][j]+r_motif[l][k] ][i][j][k]++;
		 }       
	    }
    }
 
  for(i=0;i<d*d*d;i++)
    {
     for(j=0;j<w-2;j++) 
	    {
	     for(k=j+1;k<w-1;k++) 
			{
			 for(l=k+1;l<w;l++) r_theta_triple[i][j][k][l]=r_theta_triple[i][j][k][l]/K_motif;
		    }
	    }
    }

 return;
}

//for base pair (col_i and col_j), calculate the Hamming distance between two independent 4-dimensional multinomial model and one 16-dimensional multinomial model  
double e_abs_distance(int col_i, int col_j) //col_i < col_j
{
 int i,j;
 double sum;
 sum=0.0;

 for(i=0;i<d;i++)
 {
  for(j=0;j<d;j++)
  {
   sum +=  fabs(r_theta_pair[4*i+j][col_i][col_j] - r_theta_single[i][col_i]*r_theta_single[j][col_j]); 
  }
 }

 return sum;
}

//given the motif starting position a, calculate the motif PSWM, drop the sequences which do not contain motif, and assume up to triple intra-motif dependency
//update id[i], which indicates which seqeunce contains motif
void setthetadrop_dep(int **r, int *a, int *id, double **theta, int **motif_model, 
	                  double **r_theta_single, double ***r_theta_pair, double ****r_theta_triple)
{
 int i,j,k;
 K_motif=0;
 for(i=0;i<K;i++) K_motif += id[i];

 r_motif = new int*[K_motif];
 for(i=0;i<K_motif;i++) { r_motif[i] = new int [w]; }

 k=0;
 for(i=0;i<K;i++)
	{
	 if(id[i]==1)
		{
         for(j=0;j<w;j++) {r_motif[k][j] = r[i][a[i]+j];}
	     k++;
		}
    }
 
  count_theta_single( r_motif );
  count_theta_pair( r_motif );
  count_theta_triple( r_motif );

  double ** BIC_array;
  BIC_array = new double * [w*(w-1)/2];
  for(i=0;i<w*(w-1)/2;i++) BIC_array[i] = new double [4];
  
  k=0;
  for(i=0;i<w-1;i++)
	{
     for(j=i+1;j<w;j++)
		{
	     BIC_array[k][0] = i;
		 BIC_array[k][1] = j;
		 BIC_array[k][2] = -2*(emissioncor(r_motif, i, j)-emissionindep(r_motif, i)-emissionindep(r_motif, j)) + (d*d-d*2)*log(K_motif);
		 BIC_array[k][3] = e_abs_distance(i, j);
		 k++;
	    }
    }
  
  //sort all base pairs by Hamming distance
  double temp_BIC_array; 
  for(i=0;i<w*(w-1)/2;i++)
	{
     int max=i;
     for(j=i+1;j<w*(w-1)/2;j++)
	     if(BIC_array[j][3]>BIC_array[max][3])
	       max=j;
     
    temp_BIC_array=BIC_array[i][0]; BIC_array[i][0]=BIC_array[max][0]; BIC_array[max][0]=temp_BIC_array;
	temp_BIC_array=BIC_array[i][1]; BIC_array[i][1]=BIC_array[max][1]; BIC_array[max][1]=temp_BIC_array;
	temp_BIC_array=BIC_array[i][2]; BIC_array[i][2]=BIC_array[max][2]; BIC_array[max][2]=temp_BIC_array;
	temp_BIC_array=BIC_array[i][3]; BIC_array[i][3]=BIC_array[max][3]; BIC_array[max][3]=temp_BIC_array;
    }

  //connection problem
  //reference: http://www.sph.umich.edu/csg/abecasis/class/  lecture01
  int group_index_set, group_count_max;
  int ** group_index;
  group_index = new int * [w];
  for(i=0;i<w;i++) group_index[i] = new int [2];

  for(i=0;i<w;i++)
	{
     group_index[i][0] = i;
	 group_index[i][1] = 1;
    }

  for(k=0;k<w*(w-1)/2;k++)
  {
   //model intra-motif depenedency when Hamming distance above some threshold (0.2), and complicated model results in a better BIC
   if(BIC_array[k][2]<0 && BIC_array[k][3]>=0.20) 
	  {
       if(group_index[ (int)BIC_array[k][0] ][0] == group_index[ (int)BIC_array[k][1] ][0]) continue;

       group_index_set = group_index[ (int)BIC_array[k][0] ][0];
	   group_count_max = group_index[ (int)BIC_array[k][0] ][1] + group_index[ (int)BIC_array[k][1] ][1];
      
       if(group_count_max>3) break; //only allows up to triple intra-motif dependency

       for(i=0;i<w;i++)
		   if( group_index[i][0] == group_index_set )
		       group_index[i][0] = group_index[ (int)BIC_array[k][1] ][0];
			 	
	   for(i=0;i<w;i++)
		   if( group_index[i][0] == group_index[ (int)BIC_array[k][1] ][0] )
		       group_index[i][1] = group_count_max;
	  }
  }

  //motif_model: a w*4 matrix
  //col 0: indicator of correlation structure
  //    1: independent column ~multinomial(p1,...,p4)
  //    2: pair correlation   ~multinomial(p1,...,p16)
  //    3: triple correlation ~multinomial(p1,...,p64)
  //col 1~3: cluster members
  
  //initialization: each element = -1
  for(i=0;i<w;i++)
	{
	 for(j=0;j<4;j++)
         motif_model[i][j] = -1;
    }
  
  //read in the correlation structrue
  for(i=0;i<w;i++)
	{
      if( group_index[i][1] == 1 )
		{
	     motif_model[i][0] = 1;
		 motif_model[i][1] = i;
	    }
	  else if( group_index[i][1] == 2 )
        {
	     motif_model[i][0] = 2;
		 motif_model[i][1] = i;
		 for(j=(i+1);j<w;j++)
			{
		     if(group_index[j][0] == group_index[i][0])
				{
				 group_index[j][1] = -1;
				 motif_model[i][2] = j;
				 break;
			    }
		    }
	    }
      else if( group_index[i][1] == 3 )
		{
	     motif_model[i][0] = 3;
		 motif_model[i][1] = i;
		 k=1;
		 for(j=(i+1);j<w;j++)
			{
		     if(group_index[j][0] == group_index[i][0])
				{
				 group_index[j][1] = -1;
			     motif_model[i][1+k] = j;
				 k++;
				 if(k==3) break;
			    }
		    }
	    }
    }

  //use BIC, compare one 64-dimensional multinomial model with three 16-dimesnsional multinomial model
  double BIC_triple, BIC_pair_1, BIC_pair_2, BIC_pair_3, BIC_pair_min;
  for(i=0;i<w;i++)
  {
   if(motif_model[i][0] == 3)
	  {
       BIC_triple = -2*emissiontriple(r_motif, motif_model[i][1], motif_model[i][2], motif_model[i][3])+d*d*d*log(K_motif);
       BIC_pair_1 = -2*emissioncor(r_motif, motif_model[i][1], motif_model[i][2])-2*emissionindep(r_motif, motif_model[i][3])+(d*d+d)*log(K_motif);
       BIC_pair_2 = -2*emissioncor(r_motif, motif_model[i][1], motif_model[i][3])-2*emissionindep(r_motif, motif_model[i][2])+(d*d+d)*log(K_motif);
       BIC_pair_3 = -2*emissioncor(r_motif, motif_model[i][2], motif_model[i][3])-2*emissionindep(r_motif, motif_model[i][1])+(d*d+d)*log(K_motif);

	   BIC_pair_min = BIC_pair_1;
	   if(BIC_pair_2 < BIC_pair_min) BIC_pair_min = BIC_pair_2;
	   if(BIC_pair_3 < BIC_pair_min) BIC_pair_min = BIC_pair_3;

       if(flag_triple==1)
		  {
       if(BIC_pair_min < BIC_triple)
		  {
	       if(BIC_pair_min == BIC_pair_1)
			  {
               motif_model[ motif_model[i][3] ][0] = 1;
			   motif_model[ motif_model[i][3] ][1] = motif_model[i][3];
			   motif_model[i][0] = 2;
			   motif_model[i][3] = -1;
		      }
		   else if(BIC_pair_min == BIC_pair_2)
			  {
		       motif_model[ motif_model[i][2] ][0] = 1;
			   motif_model[ motif_model[i][2] ][1] = motif_model[i][2];
			   motif_model[i][0] = 2;
			   motif_model[i][2] = motif_model[i][3];
			   motif_model[i][3] = -1;
		      }
           else if(BIC_pair_min == BIC_pair_3)
			  {
			   motif_model[ motif_model[i][2] ][0] = 2;
			   motif_model[ motif_model[i][2] ][1] = motif_model[i][2];
               motif_model[ motif_model[i][2] ][2] = motif_model[i][3];
		       motif_model[ motif_model[i][1] ][0] = 1;
			   motif_model[ motif_model[i][1] ][1] = motif_model[i][1];
               motif_model[ motif_model[i][1] ][2] = -1;
			   motif_model[ motif_model[i][1] ][3] = -1;
		      }
	      }
		  }
        else if(flag_triple==0)
		  {
		   if(BIC_pair_min == BIC_pair_1)
			  {
               motif_model[ motif_model[i][3] ][0] = 1;
			   motif_model[ motif_model[i][3] ][1] = motif_model[i][3];
			   motif_model[i][0] = 2;
			   motif_model[i][3] = -1;
		      }
		   else if(BIC_pair_min == BIC_pair_2)
			  {
		       motif_model[ motif_model[i][2] ][0] = 1;
			   motif_model[ motif_model[i][2] ][1] = motif_model[i][2];
			   motif_model[i][0] = 2;
			   motif_model[i][2] = motif_model[i][3];
			   motif_model[i][3] = -1;
		      }
           else if(BIC_pair_min == BIC_pair_3)
			  {
			   motif_model[ motif_model[i][2] ][0] = 2;
			   motif_model[ motif_model[i][2] ][1] = motif_model[i][2];
               motif_model[ motif_model[i][2] ][2] = motif_model[i][3];
		       motif_model[ motif_model[i][1] ][0] = 1;
			   motif_model[ motif_model[i][1] ][1] = motif_model[i][1];
               motif_model[ motif_model[i][1] ][2] = -1;
			   motif_model[ motif_model[i][1] ][3] = -1;
		      }
		  }
	  }
  }

  if(flag_triple==-1)
	{
     for(i=0;i<w;i++)
		{
	     motif_model[i][0]=1;
		 motif_model[i][1]=i;
		 motif_model[i][2]=-1;
		 motif_model[i][3]=-1;
	    }
    }

 //update the indicator id[i]
 double avg[K];
 for(i=0;i<K;i++)
 {
  avg[i]=0.0;

  for(j=0;j<seqlength[i]-w+1;j++)
  {
   avg[i] += exp( score_dep(r, theta, i, j, motif_model, r_theta_single, r_theta_pair, r_theta_triple) );
  }
   avg[i]=log(avg[i]/seqlength[i]);

   if(avg[i]>=0) id[i]=1;
   else if(avg[i]<0) id[i]=0;
 }
 
 //copy r_theta_single into theta 
 for(i=0;i<d;i++)
    for(j=0;j<w;j++) theta[i][j]=r_theta_single[i][j];
 
 return;
}

//gibbs sampler the motif starting position in each sequenec based on log motif score
//for sequences with motif (id[i]==1), SAMPLE[i]==1: gibbs sampler, SAMPLE[i]==0: deterministic scan
//shift to left/right one base pair, use Metropolis-Hastings algorithm to accept the shift or not
double gibbsa(int **r, double **theta, int loop, int chain_num, int ** motif_model, 
			  double **r_theta_single,  double ***r_theta_pair, double ****r_theta_triple)
{
  int i,j,k;
  double max,sump;

 if(flag_triple==-1) //flag_triple==-1, assume intra-motif independency
 {
  if(loop<LOOP*0.5) settheta(r,a);                        //first half gibbs sampler, using all sequences
  else if(loop>=LOOP*0.5) setthetadrop(r,a,id,theta);     //second half gibbs sampler, drop seqeunces without motif

  genmul((int)(K*seqprop), sampleprob[loop], K, SAMPLE);

  for(i=0;i<K;i++)
  {
   if(id[i]==1)
   {
   	if(SAMPLE[i]==1)
	{
     for(j=0;j<seqlength[i]-w+1;j++)
	 {
       if(flag_basecoverage==0) rowscore[i][j]=score(r, theta, i, j);  
       else if(flag_basecoverage==1 && loop<LOOP*0.5) rowscore[i][j]=score(r, theta, i, j) + log(1.0+rbase[i][j]);  
       else if(flag_basecoverage==1 && loop>=LOOP*0.5) rowscore[i][j]=score(r, theta, i, j); 
       else if(flag_basecoverage==2 && loop<LOOP*0.5) rowscore[i][j]=score(r, theta, i, j) - (t_dof+1)/2 * log( 1+ floor(fabs((j-rpeak[i])/bin_size)) * floor(fabs((j-rpeak[i])/bin_size)) /t_dof ) ; 
       else if(flag_basecoverage==2 && loop>=LOOP*0.5) rowscore[i][j]=score(r, theta, i, j); 

	   if(j==0) max=rowscore[i][j];
	   else if(j>0 && rowscore[i][j]>max) max=rowscore[i][j];
	 }	 
     for(j=0;j<seqlength[i]-w+1;j++) rowscore[i][j]=exp(rowscore[i][j]-max);
    
	 sump=0.0;
	 for(j=0;j<seqlength[i]-w+1;j++) sump += rowscore[i][j];
	 for(j=0;j<seqlength[i]-w+1;j++) rowscore[i][j] /= sump;
     
	 for(j=0;j<seqlength[i]-w+1;j++) 
	 {
	  if(rowscore[i][j]<0.0001) rowscore[i][j]=0.0;
	  else if(rowscore[i][j]>=0.0001) rowscore[i][j] -= 0.0001;
	 }     

     a[i] = genmulone(rowscore[i], seqlength[i]-w+1);
	}

    else if(SAMPLE[i]==0)
	{
	 for(j=0;j<seqlength[i]-w+1;j++)
	 {
      if(flag_basecoverage==0) rowscore[i][j]=score(r, theta, i, j);  
      else if(flag_basecoverage==1 && loop<LOOP*0.5) rowscore[i][j]=score(r, theta, i, j) + log(1.0+rbase[i][j]);  
      else if(flag_basecoverage==1 && loop>=LOOP*0.5) rowscore[i][j]=score(r, theta, i, j);       
      else if(flag_basecoverage==2 && loop<LOOP*0.5) rowscore[i][j]=score(r, theta, i, j) - (t_dof+1)/2 * log( 1+ floor(fabs((j-rpeak[i])/bin_size)) * floor(fabs((j-rpeak[i])/bin_size)) /t_dof ) ; 
      else if(flag_basecoverage==2 && loop>=LOOP*0.5) rowscore[i][j]=score(r, theta, i, j); 

	  if(j==0) {max=rowscore[i][j]; a[i]=j;}
	  else if(j>0 && rowscore[i][j]>max) {max=rowscore[i][j]; a[i]=j;}
	 }
	}  
   }
  }
 }
 else if(flag_triple!=-1) //flag_triple==-1, assume pair or triple intra-motif dependency
 {
  if(loop<LOOP*0.4) settheta(r,a);                                                        //first 40% gibbs sampler, using all sequences, assume intra-motif independency
  else if(loop>=LOOP*0.4 && loop<LOOP*0.8) setthetadrop(r,a,id,theta);                    //second 40% gibbs sampler, drop seqeunces without motif, assume intra-motif independency
  else if(loop>=LOOP*0.8) setthetadrop_dep(r, a, id, theta, motif_model, 
	                                       r_theta_single, r_theta_pair, r_theta_triple); //last 20% gibbs sampler, drop seqeunces without motif, assume up to triple intra-motif dependency

  genmul((int)(K*seqprop), sampleprob[loop], K, SAMPLE);

  for(i=0;i<K;i++)
  {
   if(id[i]==1)
   {
   	if(SAMPLE[i]==1)
	{
     for(j=0;j<seqlength[i]-w+1;j++)
	 {
       if(flag_basecoverage==0 && loop<LOOP*0.8) rowscore[i][j]=score(r, theta, i, j);  
	   else if(flag_basecoverage==0 && loop>=LOOP*0.8) rowscore[i][j]=score_dep(r, theta, i, j, motif_model, r_theta_single, r_theta_pair, r_theta_triple);
       else if(flag_basecoverage==1 && loop<LOOP*0.4) rowscore[i][j]=score(r, theta, i, j) + log(1.0+rbase[i][j]);  
       else if(flag_basecoverage==1 && loop>=LOOP*0.4 && loop<LOOP*0.8) rowscore[i][j]=score(r, theta, i, j); 
       else if(flag_basecoverage==1 && loop>LOOP*0.8) rowscore[i][j]=score_dep(r, theta, i, j, motif_model, r_theta_single, r_theta_pair, r_theta_triple);
       else if(flag_basecoverage==2 && loop<LOOP*0.4) rowscore[i][j]=score(r, theta, i, j) - (t_dof+1)/2 * log( 1+ floor(fabs((j-rpeak[i])/bin_size)) * floor(fabs((j-rpeak[i])/bin_size)) /t_dof ) ; 
       else if(flag_basecoverage==2 && loop>=LOOP*0.4 && loop<LOOP*0.8) rowscore[i][j]=score(r, theta, i, j); 
       else if(flag_basecoverage==2 && loop>LOOP*0.8) rowscore[i][j]=score_dep(r, theta, i, j, motif_model, r_theta_single, r_theta_pair, r_theta_triple);

	   if(j==0) max=rowscore[i][j];
	   else if(j>0 && rowscore[i][j]>max) max=rowscore[i][j];
	 }	 
     for(j=0;j<seqlength[i]-w+1;j++) rowscore[i][j]=exp(rowscore[i][j]-max);
    
	 sump=0.0;
	 for(j=0;j<seqlength[i]-w+1;j++) sump += rowscore[i][j];
	 for(j=0;j<seqlength[i]-w+1;j++) rowscore[i][j] /= sump;
     
	 for(j=0;j<seqlength[i]-w+1;j++) 
	 {
	  if(rowscore[i][j]<0.0001) rowscore[i][j]=0.0;
	  else if(rowscore[i][j]>=0.0001) rowscore[i][j] -= 0.0001;
	 }     

     a[i] = genmulone(rowscore[i], seqlength[i]-w+1);
	}

    else if(SAMPLE[i]==0)
	{
	 for(j=0;j<seqlength[i]-w+1;j++)
	 {
      if(flag_basecoverage==0 && loop<LOOP*0.8) rowscore[i][j]=score(r, theta, i, j); 
	  else if(flag_basecoverage==0 && loop>=LOOP*0.8) rowscore[i][j]=score_dep(r, theta, i, j, motif_model, r_theta_single, r_theta_pair, r_theta_triple);
      else if(flag_basecoverage==1 && loop<LOOP*0.4 ) rowscore[i][j]=score(r, theta, i, j) + log(1.0+rbase[i][j]);  
      else if(flag_basecoverage==1 && loop>=LOOP*0.4 && loop<LOOP*0.8) rowscore[i][j]=score(r, theta, i, j);       
      else if(flag_basecoverage==1 && loop>LOOP*0.8) rowscore[i][j]=score_dep(r, theta, i, j, motif_model, r_theta_single, r_theta_pair, r_theta_triple);
	  else if(flag_basecoverage==2 && loop<LOOP*0.4) rowscore[i][j]=score(r, theta, i, j) - (t_dof+1)/2 * log( 1+ floor(fabs((j-rpeak[i])/bin_size)) * floor(fabs((j-rpeak[i])/bin_size)) /t_dof ) ; 
      else if(flag_basecoverage==2 && loop>=LOOP*0.4 && loop<LOOP*0.8) rowscore[i][j]=score(r, theta, i, j); 
      else if(flag_basecoverage==2 && loop>LOOP*0.8) rowscore[i][j]=score_dep(r, theta, i, j, motif_model, r_theta_single, r_theta_pair, r_theta_triple);

	  if(j==0) {max=rowscore[i][j]; a[i]=j;}
	  else if(j>0 && rowscore[i][j]>max) {max=rowscore[i][j]; a[i]=j;}
	 }
	}  
   }
  }
 }

  //begin shift to left/right one base pair, use Metropolis-Hastings algorithm to accept the shift or not
  double total=0.0, value=0.0;

  for(i=0;i<K;i++)
  {
   if(id[i]==1)
   {
	if(flag_triple==-1) value += score(r, theta, i, a[i]); 
    else if(flag_triple!=-1 && loop<LOOP*0.8) value += score(r, theta, i, a[i]);
	else if(flag_triple!=-1 && loop>=LOOP*0.8) value += score_dep(r, theta, i, a[i], motif_model, r_theta_single, r_theta_pair, r_theta_triple);  
	total++;
   }
  }

  int a_MH_left[K], a_end_left[K], a_MH_right[K], a_end_right[K];
  double indicate, diff_left, diff_right;

	 for(i=0;i<K;i++)
	 {
	  a_MH_left[i]=a[i]-1;
      if(a_MH_left[i]<0) a_MH_left[i]=0;
	  a_end_left[i]=a[i]+w-1;
     }
     diff_left=score_single(r, a_MH_left)-score_single(r, a_end_left);   
	 
	 for(i=0;i<K;i++)
	 {
	  a_MH_right[i]=a[i];
	  a_end_right[i]=a[i]+w;
	  if(a_end_right[i]==seqlength[i]) a_end_right[i]=seqlength[i]-1;
     }
     diff_right=score_single(r, a_end_right)-score_single(r, a_MH_right); 

     if(diff_left>diff_right)
	 {
		if(diff_left>0)
		{
	     for(i=0;i<K;i++) a[i]=a_MH_left[i];
		 return( (value+diff_left)/total );
	    }
	    else if(diff_left<=0)
		{
	     indicate=genunf(0,1);
		 if(indicate<exp(diff_left))
			{
		     for(i=0;i<K;i++) a[i]=a_MH_left[i];
		     return( (value+diff_left)/total );
		    }
		 else if(indicate>=exp(diff_left))
			{
		     return (value/total);
		    }
	    }
	 }
	 else if(diff_left<=diff_right)
     {  
        if(diff_right>0)
		{
	     for(i=0;i<K;i++)
			 { 
			  if(a_MH_right[i]<seqlength[i]-w) a[i]=a_MH_right[i]+1; 
			  else if(a_MH_right[i]==seqlength[i]-w) a[i]=a_MH_right[i]; 
		     }
		 return( (value+diff_right)/total );
	    }
	    else if(diff_right<=0)
		{
	     indicate=genunf(0,1);
		 if(indicate<exp(diff_right))
			{
		     for(i=0;i<K;i++)
			 { 
			  if(a_MH_right[i]<seqlength[i]-w) a[i]=a_MH_right[i]+1; 
			  else if(a_MH_right[i]==seqlength[i]-w) a[i]=a_MH_right[i]; 
		     }
		     return( (value+diff_right)/total );
		    }
		 else if(indicate>exp(diff_right))
			{
             return (value/total);		 
		    }
         }
	 }
   //end shift to left/right one base pair, use Metropolis-Hastings algorithm to accept the shift or not
 }

//in each chain, keep the motif starting position, motif inidcator corresponding to highest log motif score
double gibbsmotif(int **r, int chain_num)
{
 int i,j,k,l,loop;
 double t, tmax;

 if(flag_basecoverage==0) setlabel();
 else if(flag_basecoverage==1) setlabel_basecoverage();
 else if(flag_basecoverage==2) setlabel_peaklocation();

 for(i=0;i<K;i++) id[i]=1;

 for(loop=0;loop<LOOP;loop++)
 {
  t=gibbsa(r,theta,loop,chain_num, motif_model, r_theta_single, r_theta_pair, r_theta_triple);

  logplot[loop][countchain]=t;

 if(flag_triple==-1)
 {
  if(loop==0) 
  {
	tmax=t;
    for(i=0;i<K;i++) {atemp[i]=a[i]; idtemp[i]=id[i];}
	for(i=0;i<d;i++) { for(j=0;j<w;j++) thetatemp[i][j]=theta[i][j];  }
  }

  else if(loop>0 && t>tmax)
  {
    tmax=t;
	for(i=0;i<K;i++) {atemp[i]=a[i]; idtemp[i]=id[i];}
	for(i=0;i<d;i++) { for(j=0;j<w;j++) thetatemp[i][j]=theta[i][j]; }
  }
 }
 else if(flag_triple!=-1)
 {
     //output: motif_model motif_model_temp motif_model_final
	 //        r_theta_single r_theta_single_temp r_theta_single_final
     //        r_theta_pair r_theta_pair_temp r_theta_pair_final
	 //        r_theta_triple r_theta_triple_temp r_theta_triple_final
  if(loop==0) 
  {
	tmax=t;
    for(i=0;i<K;i++) {atemp[i]=a[i]; idtemp[i]=id[i];}
	for(i=0;i<d;i++) { for(j=0;j<w;j++) thetatemp[i][j]=theta[i][j];  }
  }
  else if(loop>0 && loop<LOOP*0.8 && t>tmax)
  {
    tmax=t;
	for(i=0;i<K;i++) {atemp[i]=a[i]; idtemp[i]=id[i];}
	for(i=0;i<d;i++) { for(j=0;j<w;j++) thetatemp[i][j]=theta[i][j]; }
  }
  else if(loop>=LOOP*0.8 && t>tmax)
  {
    tmax=t;
    for(i=0;i<K;i++) {atemp[i]=a[i]; idtemp[i]=id[i];}
	for(i=0;i<d;i++) { for(j=0;j<w;j++) thetatemp[i][j]=theta[i][j]; }
    
    for(i=0;i<w;i++) { for(j=0;j<d;j++) motif_model_temp[i][j] = motif_model[i][j]; }
    for(i=0;i<d;i++) { for(j=0;j<w;j++) r_theta_single_temp[i][j] = r_theta_single[i][j]; }    
    for(i=0;i<d*d;i++) { for(j=0;j<w-1;j++) { for(k=j+1;k<w;k++) r_theta_pair_temp[i][j][k] = r_theta_pair[i][j][k]; } }
    for(i=0;i<d*d*d;i++) { for(j=0;j<w-2;j++) { for(k=j+1;k<w-1;k++) { for(l=k+1;l<w;l++) r_theta_triple_temp[i][j][k][l] = r_theta_triple[i][j][k][l]; } } }
  }

 }

 }
 return(tmax);
}

  int main(int argc, char **argv)
{ 
  time_t time_start, time_end;
  struct tm * timeinfo_start, * timeinfo_end;

  time(&time_start);
  timeinfo_start=localtime(&time_start);
  
  FILE * s_time;
  s_time=fopen("time.txt","w");
  fprintf(s_time, "HMS start time:\t%s\n", asctime(timeinfo_start));

  int i,j,k,l,m;
  double value,valuetemp;

  char * pch=NULL;
  char str[max_length];

if(argc<18)
{
printf("syntax: ./hms -i input_seq -w width -dna d_acgt -iteration no_of_iteration -chain no_of_chain -seqprop prop_of_random_sample -strand no_of_strand -base basecoverage_profile -dep dependency\n");
printf("\n");
printf("9 options need to be specified \n\tINPUTFILE,\n\twidth,\n\td_acgt,\n\tno_of_loop,\n\tno_of_chain,\n\tseqprop,\n\tstrand,\n\tprior,\n\tdependency. \n");
exit(0);
}

for(i=1;i<argc-1;i++)
{
 if(strcmp(argv[i],"-i")==0){name_input_sequences=i+1;i++;}
 else if(strcmp(argv[i],"-w")==0){w=atoi(argv[i+1]);i++;}
 else if(strcmp(argv[i],"-dna")==0){d=atoi(argv[i+1]);i++;}
 else if(strcmp(argv[i],"-iteration")==0){LOOP=atoi(argv[i+1]);i++;}
 else if(strcmp(argv[i],"-chain")==0){chain=atoi(argv[i+1]);i++;}
 else if(strcmp(argv[i],"-seqprop")==0){seqprop=atof(argv[i+1]);i++;}
 else if(strcmp(argv[i],"-strand")==0){flag_strand=atoi(argv[i+1]);i++;}
 else if(strcmp(argv[i],"-base")==0){name_input_basecoverage=i+1; flag_basecoverage=1; i++;}
 else if(strcmp(argv[i],"-nobase")==0){flag_basecoverage=0;}
 else if(strcmp(argv[i],"-peaklocation")==0){name_input_peaklocation=i+1; flag_basecoverage=2; i++;}
 else if(strcmp(argv[i],"-t_dof")==0){t_dof=atof(argv[i+1]);i++;}
 else if(strcmp(argv[i],"-dep")==0){flag_triple=atoi(argv[i+1])-2;i++;}
}

if(flag_triple!=-1)
{
  motif_model = new int * [w];
  for(i=0;i<w;i++) motif_model[i] = new int [d];
  motif_model_temp = new int * [w];
  for(i=0;i<w;i++) motif_model_temp[i] = new int [d];
  motif_model_final = new int * [w];
  for(i=0;i<w;i++) motif_model_final[i] = new int [d];

  r_theta_single = new double*[d];
  for(i=0;i<d;i++) {r_theta_single[i] = new double [w]; }
  r_theta_single_temp = new double*[d];
  for(i=0;i<d;i++) {r_theta_single_temp[i] = new double [w]; }
  r_theta_single_final = new double*[d];
  for(i=0;i<d;i++) {r_theta_single_final[i] = new double [w]; }

  r_theta_pair = new double ** [d*d];
  for(i=0;i<(d*d);i++) 
	  {
	   r_theta_pair[i] = new double * [w];
	   for(j=0;j<w;j++) r_theta_pair[i][j] = new double [w];
	  }
  r_theta_pair_temp = new double ** [d*d];
  for(i=0;i<(d*d);i++) 
	  {
	   r_theta_pair_temp[i] = new double * [w];
	   for(j=0;j<w;j++) r_theta_pair_temp[i][j] = new double [w];
	  }
  r_theta_pair_final = new double ** [d*d];
  for(i=0;i<(d*d);i++) 
	  {
	   r_theta_pair_final[i] = new double * [w];
	   for(j=0;j<w;j++) r_theta_pair_final[i][j] = new double [w];
	  }

  r_theta_triple = new double *** [d*d*d];
  for(i=0;i<(d*d*d);i++)
	{
      r_theta_triple[i] = new double ** [w];
	  for(j=0;j<w;j++)
		{
	     r_theta_triple[i][j] = new double * [w];
		 for(k=0;k<w;k++) r_theta_triple[i][j][k] = new double [w];
	    }
    }
  r_theta_triple_temp = new double *** [d*d*d];
  for(i=0;i<(d*d*d);i++)
	{
      r_theta_triple_temp[i] = new double ** [w];
	  for(j=0;j<w;j++)
		{
	     r_theta_triple_temp[i][j] = new double * [w];
		 for(k=0;k<w;k++) r_theta_triple_temp[i][j][k] = new double [w];
	    }
    }
  r_theta_triple_final = new double *** [d*d*d];
  for(i=0;i<(d*d*d);i++)
	{
      r_theta_triple_final[i] = new double ** [w];
	  for(j=0;j<w;j++)
		{
	     r_theta_triple_final[i][j] = new double * [w];
		 for(k=0;k<w;k++) r_theta_triple_final[i][j][k] = new double [w];
	    }
    }
}

/* begin to count the number of input sequences */
  i=0;
  fstream file_op(argv[ name_input_sequences ],ios::in);
  while(!file_op.eof() ) 
  {
   file_op.getline(str,max_length); if (str[0] != '>') break;
   file_op.getline(str,max_length);
   for(j=0;j<max_length;j++)
	  {
       if(str[j]!='A' && str[j]!= 'C' && str[j]!='G' && str[j]!='T' && str[j]!='N'
	   && str[j]!='a' && str[j]!= 'c' && str[j]!='g' && str[j]!='t' && str[j]!='n')
		  {
	       break;
		  }
      }
   i++;
  }         
  file_op.close();

if(flag_strand==2)
{
  Khalf=i;
  K=i*2;
}
else if(flag_strand==1)
{
 K=i;
}
/* end to count the number of input sequences */

  logplot = new double *[LOOP];
  for(i=0;i<LOOP;i++) { logplot[i] = new double[chain]; }

if(flag_strand==2)
{
  halfseqlength = new int[Khalf];
  seqlength     = new int[K];
}
else if(flag_strand==1)
{
 seqlength = new int[K];
}

  a             = new int[K];
  atemp         = new int[K];
  afinal        = new int[K];
  id            = new int[K];
  idtemp        = new int[K];
  idfinal       = new int[K];
  SAMPLE        = new long[K];

  sampleprob = new double*[LOOP];
  for(i=0;i<LOOP;i++) { sampleprob[i] = new double[K]; }

  theta0 = new double[d];

/* begin read theta0 */
	i=0;
	j=0;
    fstream file_op_theta0("theta0.txt",ios::in);
    while(!file_op_theta0.eof() ) 
    {
     file_op_theta0.getline(str,max_length); 
	 theta0[i]=atof(str);
     i++;	  
	}         
	file_op_theta0.close();
/* end read theta0 */

  theta1 = new double*[d];
  for(i=0;i<d;i++) { theta1[i] = new double[d]; }

/* begin read theta1 */
	i=0;
	j=0;
    fstream file_op_theta1("theta1.txt",ios::in);
    while(!file_op_theta1.eof() ) 
    {
     file_op_theta1.getline(str,max_length); 
     pch=strtok(str,"	");
     while(pch!=NULL)
      {
       theta1[i][j]=atof(pch);
	   pch=strtok(NULL, "	");
	   j++;
	  }
	    j=0;
	    i++;	  
	}         
	file_op_theta1.close();
/* end read theta1 */

  theta2 = new double*[d];
  for(i=0;i<d;i++) { theta2[i] = new double[d*d]; }

/* begin read theta2 */
	i=0;
	j=0;
    fstream file_op_theta2("theta2.txt",ios::in);
    while(!file_op_theta2.eof() ) 
    {
     file_op_theta2.getline(str,max_length); 
     pch=strtok(str,"	");
     while(pch!=NULL)
      {
       theta2[i][j]=atof(pch);
	   pch=strtok(NULL, "	");
	   j++;
	  }
	    j=0;
	    i++;	  
	}         
	file_op_theta2.close();
/* end read theta2 */

  theta3 = new double*[d];
  for(i=0;i<d;i++) { theta3[i] = new double[d*d*d]; }

/* begin read theta3 */
	i=0;
	j=0;
    fstream file_op_theta3("theta3.txt",ios::in);
    while(!file_op_theta3.eof() ) 
    {
     file_op_theta3.getline(str,max_length); 
     pch=strtok(str,"	");
     while(pch!=NULL)
      {
       theta3[i][j]=atof(pch);
	   pch=strtok(NULL, "	");
	   j++;
	  }
	    j=0;
	    i++;	  
	}         
	file_op_theta3.close();
/* end read theta3 */

  theta = new double*[d];
  for(i=0;i<d;i++) { theta[i] = new double[w]; }

  thetatemp = new double*[d];
  for(i=0;i<d;i++) { thetatemp[i] = new double[w]; }

  thetafinal = new double*[d];
  for(i=0;i<d;i++) { thetafinal[i] = new double[w]; }

/* begin to count the number of base in each input sequences */
  i=0;
  fstream file_op1(argv[ name_input_sequences ],ios::in);
  while(!file_op1.eof() ) 
  {
   file_op1.getline(str,max_length); if (str[0] != '>') break; 
   file_op1.getline(str,max_length);
   for(j=0;j<max_length;j++)
	  {
       if(str[j]!='A' && str[j]!= 'C' && str[j]!='G' && str[j]!='T' && str[j]!='N'
	   && str[j]!='a' && str[j]!= 'c' && str[j]!='g' && str[j]!='t' && str[j]!='n')
		  {
		   if(flag_strand==2)  halfseqlength[i]=j;
	       else if(flag_strand==1) seqlength[i]=j;
		   break;
		  }
      }
   i++;
  }         
  file_op1.close();
/* end to count the number of base in each input sequences */

if(flag_strand==2)
{
  rinput = new int*[Khalf];
  for(i=0;i<Khalf;i++) { rinput[i] = new int[ halfseqlength[i] ]; }

  rinverse = new int*[Khalf];
  for(i=0;i<Khalf;i++) { rinverse[i] = new int[ halfseqlength[i] ]; }
 
  r = new int*[K];
  for(i=0;i<K;i++) { r[i] = new int[ halfseqlength[i/2] ]; }

  rbackground = new double*[K];
  for(i=0;i<K;i++) { rbackground[i] = new double[ halfseqlength[i/2] ]; }

  IX = new long*[K];
  for(i=0;i<K;i++) { IX[i] = new long[ halfseqlength[i/2] ]; }

  rowscore = new double*[K];
  for(i=0;i<K;i++) { rowscore[i] = new double[ halfseqlength[i/2]-w+1 ]; }
}
else if(flag_strand==1)
{
  r = new int*[K];
  for(i=0;i<K;i++) { r[i] = new int[ seqlength[i] ]; }

  rbackground = new double*[K];
  for(i=0;i<K;i++) { rbackground[i] = new double[ seqlength[i] ]; }

  IX = new long*[K];
  for(i=0;i<K;i++) { IX[i] = new long[ seqlength[i] ]; }

  rowscore = new double*[K];
  for(i=0;i<K;i++) { rowscore[i] = new double[ seqlength[i]-w+1 ]; }
}

//read in the input file, transfer a, c, g, t into 0, 1, 2, 3.
if(flag_strand==2)
{
  i=0;
  fstream file_op2(argv[ name_input_sequences ],ios::in);
  while(!file_op2.eof() ) 
  {
   file_op2.getline(str,max_length); if (str[0] != '>') break;
   file_op2.getline(str,max_length); 
   for(j=0;j<halfseqlength[i];j++)
	   {
             if(str[j]=='a' || str[j]=='A') rinput[i][j]=0; // 'a' or 'A' -> 0
        else if(str[j]=='c' || str[j]=='C') rinput[i][j]=1; // 'c' or 'C' -> 1
        else if(str[j]=='g' || str[j]=='G') rinput[i][j]=2; // 'g' or 'G' -> 2
        else if(str[j]=='t' || str[j]=='T') rinput[i][j]=3; // 't' or 'T' -> 3
		else if(str[j]=='n' || str[j]=='N') rinput[i][j]=(int)genunf(0,1)*4;
       }
   i++;	  
  }         
  file_op2.close();
}
else if(flag_strand==1)
{
  i=0;
  fstream file_op2(argv[ name_input_sequences ],ios::in);
  while(!file_op2.eof() ) 
  {
   file_op2.getline(str,max_length); if (str[0] != '>') break;
   file_op2.getline(str,max_length); 
   for(j=0;j<seqlength[i];j++)
	   {
             if(str[j]=='a' || str[j]=='A') r[i][j]=0; // 'a' or 'A' -> 0
        else if(str[j]=='c' || str[j]=='C') r[i][j]=1; // 'c' or 'C' -> 1
        else if(str[j]=='g' || str[j]=='G') r[i][j]=2; // 'g' or 'G' -> 2
        else if(str[j]=='t' || str[j]=='T') r[i][j]=3; // 't' or 'T' -> 3
		else if(str[j]=='n' || str[j]=='N') r[i][j]=(int)genunf(0,1)*4;
       }
   i++;	  
  }         
  file_op2.close();
}

//read in sequencing depth file
if(flag_strand==2)
{
if(flag_basecoverage==1)
{
  rbaseinput = new double*[Khalf];
  for(i=0;i<Khalf;i++) { rbaseinput[i] = new double[ halfseqlength[i] ]; }

  rbaseinverse = new double*[Khalf];
  for(i=0;i<Khalf;i++) { rbaseinverse[i] = new double[ halfseqlength[i] ]; }
 
  rbase = new double*[K];
  for(i=0;i<K;i++) { rbase[i] = new double[ halfseqlength[i/2] ]; }

 	i=0;
	j=0;
    fstream file_op_rbaseinput(argv[ name_input_basecoverage ],ios::in);
    while(!file_op_rbaseinput.eof() && i<Khalf) 
    {	
     file_op_rbaseinput.getline(str,max_length);   
	 file_op_rbaseinput.getline(str,max_length);   
     pch=strtok(str," ");
     while(pch!=NULL)
      {
       if(atof(pch)>0.0) {rbaseinput[i][j]=atof(pch); j++;}
	   pch=strtok(NULL, " ");	   
      }
	    j=0;
	    i++;	  
	}         
	file_op_rbaseinput.close();
}
else if(flag_basecoverage==2)
{
    rpeakinput   = new int [Khalf];
	rpeakinverse = new int [Khalf];
	rpeak        = new int [K];

	i=0;
    fstream file_op_rpeak(argv[ name_input_peaklocation ],ios::in);
    while(!file_op_rpeak.eof() ) 
    {
     file_op_rpeak.getline(str,max_length); 
	 rpeakinput[i]=atoi(str);
     i++;	  
	}         
	file_op_rpeak.close();
}
}
else if(flag_strand==1)
{
if(flag_basecoverage==1)
{
  rbase = new double*[K];
  for(i=0;i<K;i++) { rbase[i] = new double[ seqlength[i] ]; }

 	i=0;
	j=0;
    fstream file_op_rbaseinput(argv[ name_input_basecoverage ],ios::in);
    while(!file_op_rbaseinput.eof() && i<K) 
    {	
     file_op_rbaseinput.getline(str,max_length);   
	 file_op_rbaseinput.getline(str,max_length);   
     pch=strtok(str," ");
     while(pch!=NULL)
      {
       if(atof(pch)>0.0) {rbase[i][j]=atof(pch); j++;}
	   pch=strtok(NULL, " ");	   
      }
	    j=0;
	    i++;	  
	}         
	file_op_rbaseinput.close();
}
else if(flag_basecoverage==2)
{
	rpeak = new int [K];

	i=0;
    fstream file_op_rpeak(argv[ name_input_peaklocation ],ios::in);
    while(!file_op_rpeak.eof() ) 
    {
     file_op_rpeak.getline(str,max_length); 
	 rpeak[i]=atoi(str);
     i++;	  
	}         
	file_op_rpeak.close();
}
}

if(flag_strand==2)
{
  for(i=0;i<Khalf;i++)
	{
     seqlength[i*2]=halfseqlength[i];
	 seqlength[i*2+1]=halfseqlength[i];
    }

  setrinverse(rinput);

if(flag_basecoverage==1) { setrbaseinverse(rbaseinput); }
if(flag_basecoverage==2) { setpeakinverse(rpeakinput); }
  
  combine(rinput,rinverse);
  
if(flag_basecoverage==1) { combinebase(rbaseinput,rbaseinverse); }
if(flag_basecoverage==2) { combinepeak(rpeakinput,rpeakinverse); }
}

  setsampleprob();

//calculate 3-order markov background model
  for(i=0;i<K;i++)
  {
   for(j=0;j<seqlength[i];j++)
   {
    if(j==0) rbackground[i][j]=theta0[ r[i][j] ];   
	else if(j==1) rbackground[i][j]=theta1[ r[i][j] ][ r[i][j-1] ];
	else if(j==2) rbackground[i][j]=theta2[ r[i][j] ][ r[i][j-2] + 4*r[i][j-1] ];
	else if(j>=3) rbackground[i][j]=theta3[ r[i][j] ][ r[i][j-3] + 4*r[i][j-2] + 16*r[i][j-1] ];
   } 
  }

//for different parallel chains, keep the motif starting position 
//and motif indicator corresponding to the highest log motif ratio
  countchain=0;
  int I;
  for(I=0;I<chain;I++)
  {
	valuetemp=gibbsmotif(r,I);

 	countchain++;

   if(flag_triple==-1)
   {
	if(I==0) 
	{
	 value=valuetemp;
     for(m=0;m<K;m++) {afinal[m]=atemp[m]; idfinal[m]=idtemp[m];}
	 for(m=0;m<d;m++) { for(j=0;j<w;j++) thetafinal[m][j]=thetatemp[m][j];  }
	}

	if(I>0 && valuetemp>value)
	{
	  value=valuetemp;
	  for(m=0;m<K;m++) {afinal[m]=atemp[m]; idfinal[m]=idtemp[m];}
	  for(m=0;m<d;m++) { for(j=0;j<w;j++) thetafinal[m][j]=thetatemp[m][j];  }
	}
   }
   else if(flag_triple!=-1)
	{
     //output: motif_model motif_model_temp motif_model_final
	 //        r_theta_single r_theta_single_temp r_theta_single_final
     //        r_theta_pair r_theta_pair_temp r_theta_pair_final
	 //        r_theta_triple r_theta_triple_temp r_theta_triple_final
    if(I==0) 
	{
	 value=valuetemp;
     for(m=0;m<K;m++) {afinal[m]=atemp[m]; idfinal[m]=idtemp[m];}
	 for(m=0;m<d;m++) { for(j=0;j<w;j++) thetafinal[m][j]=thetatemp[m][j];  }

	 for(i=0;i<w;i++) { for(j=0;j<d;j++) motif_model_final[i][j] = motif_model_temp[i][j]; }
     for(i=0;i<d;i++) { for(j=0;j<w;j++) r_theta_single_final[i][j] = r_theta_single_temp[i][j]; }    
     for(i=0;i<d*d;i++) { for(j=0;j<w-1;j++) { for(k=j+1;k<w;k++) r_theta_pair_final[i][j][k] = r_theta_pair_temp[i][j][k]; } }
     for(i=0;i<d*d*d;i++) { for(j=0;j<w-2;j++) { for(k=j+1;k<w-1;k++) { for(l=k+1;l<w;l++) r_theta_triple_final[i][j][k][l] = r_theta_triple_temp[i][j][k][l]; } } }
	}

	if(I>0 && valuetemp>value)
	{
	  value=valuetemp;
	  for(m=0;m<K;m++) {afinal[m]=atemp[m]; idfinal[m]=idtemp[m];}
	  for(m=0;m<d;m++) { for(j=0;j<w;j++) thetafinal[m][j]=thetatemp[m][j];  }

	  for(i=0;i<w;i++) { for(j=0;j<d;j++) motif_model_final[i][j] = motif_model_temp[i][j]; }
      for(i=0;i<d;i++) { for(j=0;j<w;j++) r_theta_single_final[i][j] = r_theta_single_temp[i][j]; }    
      for(i=0;i<d*d;i++) { for(j=0;j<w-1;j++) { for(k=j+1;k<w;k++) r_theta_pair_final[i][j][k] = r_theta_pair_temp[i][j][k]; } }
      for(i=0;i<d*d*d;i++) { for(j=0;j<w-2;j++) { for(k=j+1;k<w-1;k++) { for(l=k+1;l<w;l++) r_theta_triple_final[i][j][k][l] = r_theta_triple_temp[i][j][k][l]; } } }
	}
    }

	printf("The %d th sampler\n", I+1);
  }

/*re adjust thetafinal*/
double count_sum_final;
double *count_final;
count_final = new double[d];
 
 for(j=0;j<w;j++)
 {
  count_sum_final=0.0;
  for(i=0;i<d;i++) count_final[i]=0.0;

  for(i=0;i<K;i++)
  {
	if(idfinal[i]==1)
	{
	 count_final[ r[i][afinal[i]+j] ]++;
	}
  }

  for(i=0;i<d;i++) count_sum_final += count_final[i];
  for(i=0;i<d;i++) thetafinal[i][j]=count_final[i]/count_sum_final;
 }
/***********************/

  FILE *s;
  s=fopen("logplot.txt","w");
  for(i=0;i<LOOP;i++)
  {
    for(j=0;j<chain;j++) fprintf(s, "%lf ", logplot[i][j]);
	fprintf(s, "\n");
  }
  fclose(s);

  s=fopen("thetafinal.txt","w");
  for(i=0;i<d;i++)
  {
    for(j=0;j<w;j++) fprintf(s, "%lf ", thetafinal[i][j]);
	fprintf(s, "\n");
  }
  fclose(s);

  if(flag_triple!=-1)
	{
     s=fopen("motif_model.txt","w");
     fprintf(s, "Size	Memeber\n");
     for(i=0;i<w;i++)
	   {   
	    if(motif_model[i][0]!=-1) fprintf(s, "%d	", motif_model[i][0]);
        for(j=1;j<4;j++)
		  {
	       if(motif_model[i][j]!=-1) fprintf(s, "%d	", motif_model[i][j]+1);
	      }
	       if(motif_model[i][0]!=-1) fprintf(s, "\n");
       }
      fclose(s);
  
  double ** theta_single_output, ** theta_pair_output, ** theta_triple_output;
  theta_single_output = new double * [d];
  for(i=0;i<d;i++) theta_single_output[i] = new double [w];

  theta_pair_output = new double * [d*d];
  for(i=0;i<d*d;i++) theta_pair_output[i] = new double [w];

  theta_triple_output = new double * [d*d*d];
  for(i=0;i<d*d*d;i++) theta_triple_output[i] = new double [w];

//initialization
  for(i=0;i<d;i++)
	{
     for(j=0;j<w;j++) theta_single_output[i][j]=0.0;
    }
  
  for(i=0;i<d*d;i++)
	{
     for(j=0;j<w;j++) theta_pair_output[i][j]=0.0;
    }

  for(i=0;i<d*d*d;i++)
	{
     for(j=0;j<w;j++) theta_triple_output[i][j]=0.0;
    }

//save model correlation structure
  for(i=0;i<w;i++)
	{
     if(motif_model[i][0] == 1)
		{
	     for(k=0;k<d;k++) theta_single_output[k][ motif_model[i][1] ] = r_theta_single_final[k][ motif_model[i][1] ];
	    }
	 else if(motif_model[i][0] == 2)
		{
	     for(k=0;k<d*d;k++) theta_pair_output[k][ motif_model[i][1] ] = r_theta_pair_final[k][ motif_model[i][1] ][ motif_model[i][2] ]; 
	    }
	 else if(motif_model[i][0] == 3)
		{
    	 for(k=0;k<d*d*d;k++) theta_triple_output[k][ motif_model[i][1] ] = r_theta_triple_final[k][ motif_model[i][1] ][ motif_model[i][2] ][ motif_model[i][3] ]; 
	    }
    }

  s=fopen("theta_single_output.txt","w");
  for(i=0;i<d;i++)
	{
     for(j=0;j<w;j++) fprintf(s, "%.6lf	", theta_single_output[i][j]);
	 fprintf(s, "\n");
    }
  fclose(s);

  s=fopen("theta_pair_output.txt","w");
  for(i=0;i<d*d;i++)
	{
     for(j=0;j<w;j++) fprintf(s, "%.6lf	", theta_pair_output[i][j]);
	 fprintf(s, "\n");
    }
  fclose(s);

  s=fopen("theta_triple_output.txt","w");
  for(i=0;i<d*d*d;i++)
	{
     for(j=0;j<w;j++) fprintf(s, "%.6lf	", theta_triple_output[i][j]);
	 fprintf(s, "\n");
    }
  fclose(s);

    }

  s=fopen("motifresult.txt","w");
  for(i=0;i<K;i++)
  { 
   if(flag_strand==2)
	{
	 fprintf(s, "seq_%d	", i/2+1);
     if(i%2==0) fprintf(s, "+	");
	 else if(i%2==1) fprintf(s, "-	");
	}
   else if(flag_strand==1)
	{
     fprintf(s, "seq_%d	", i+1);
    }
	 
	 fprintf(s, "%d	", idfinal[i]);

	 if(idfinal[i]==1) fprintf(s, "%d	", afinal[i]);
     else if(idfinal[i]==0) fprintf(s, "-1	");

     for(j=0;j<w;j++)
	  {
             if(r[i][afinal[i]+j]==0) fprintf(s, "A");
		else if(r[i][afinal[i]+j]==1) fprintf(s, "C");
		else if(r[i][afinal[i]+j]==2) fprintf(s, "G");
		else if(r[i][afinal[i]+j]==3) fprintf(s, "T");
	  }
	  fprintf(s, "\n");
  }
  fclose(s);

  s=fopen("motifhits.txt","w");
  for(i=0;i<K;i++)
	{
     if(idfinal[i]==1)
		{
	       for(j=0;j<w;j++)
	       {
             if(r[i][afinal[i]+j]==0) fprintf(s, "A"); 
		else if(r[i][afinal[i]+j]==1) fprintf(s, "C"); 
		else if(r[i][afinal[i]+j]==2) fprintf(s, "G");
		else if(r[i][afinal[i]+j]==3) fprintf(s, "T");
	        }
           fprintf(s, "\n");
	    }
    }
  fclose(s);

  time(&time_end);
  timeinfo_end=localtime(&time_end);

  fprintf(s_time, "HMS end time:\t%s\n", asctime(timeinfo_end));
  fprintf(s_time, "HMS run time:\t%d\tseconds\n", (int)(time_end-time_start));
  fclose(s_time);

  printf("MOTIF ENDs! \n");

return (0);
}
